/* billmove.c
**
**    physics of the billard system to calculate timestep
**    Copyright (C) 2001  Florian Berger
**    Email:  harpin_floh@yahoo.de,  florian.berger@jk.uni-linz.ac.at
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License Version 2 as
**    published by the Free Software Foundation;
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

#include "billmove.h"
#include "math.h"
#include "stdlib.h"
#include "stdio.h"

static struct{
    int out_half;
    int out_full;
    int out_white;
    int out_black;
} move_log;


void BM_reset_move_info()
{
    move_log.out_half=0;
    move_log.out_full=0;
    move_log.out_white=0;
    move_log.out_black=0;
}

int BM_get_balls_out_half()  { return move_log.out_half; }

int BM_get_balls_out_full()  { return move_log.out_full; }

int BM_get_balls_out_total() { return move_log.out_full+move_log.out_half; }

int BM_get_white_out() { return move_log.out_white; }



int my_rand(nr)
{
    return rand()%nr;
}


static void calc_rand_table(int * rnd, int nr)
{
    int i,i1,dummy;
    for(i=0;i<nr;i++){
        rnd[i]=i;
    }
    for(i=0;i<nr;i++){
        i1    = my_rand(nr);
        dummy = rnd[i];
        rnd[i] = rnd[i1];
        rnd[i1] = dummy;
    }
}


BMfloat plane_dist(VMvect r, VMvect rp, VMvect n)
{ return( vec_mul(vec_diff(r,rp),n) ); }


#ifdef USE_ADV_BORDER
int inrange_advborder( BallType *b, BorderType *w )
{
    VMvect r,dr,dr1,dr2,dr3,n;
    BMfloat d,dra;
    if       (w->pnr==3){
        dr1 = vec_diff(w->r2,w->r1);
        dr2 = vec_diff(w->r3,w->r2);
        dr3 = vec_diff(w->r1,w->r3);
        n   = vec_unit(vec_cross(dr1,dr2));
        return( plane_dist( b->r, w->r1, vec_unit(vec_cross(n,dr1)) ) >= 0.0 &&
                plane_dist( b->r, w->r2, vec_unit(vec_cross(n,dr2)) ) >= 0.0 &&
                plane_dist( b->r, w->r3, vec_unit(vec_cross(n,dr3)) ) >= 0.0 );
    } else if(w->pnr==2){
        r   = vec_diff(b->r,w->r1);
        dr  = vec_diff(w->r2,w->r1);
        dra = vec_abs(dr);
        d   = vec_mul(r,dr)/dra;
        return (d>=0.0 && d<dra);
    } else if(w->pnr==1){
        return 1;
    }
    return 1;
}

BMfloat ball_advborder_dist( BallType *b, BorderType *w )
{
    VMvect r,dr;
    if(inrange_advborder(b,w)){
        if       (w->pnr==3){
            return vec_mul(vec_diff(b->r,w->r1),w->n);
        } else if(w->pnr==2){
            r=vec_diff(b->r,w->r1);
            dr=vec_diff(w->r2,w->r1);
            return vec_abs(vec_diff(r,vec_proj(r,dr)));
        } else if(w->pnr==1){
            return vec_abs(vec_diff(b->r,w->r1));
        }
    } else {
        return -1.0E20;
    }
    return -1.0E20;
}
#endif


#ifdef USE_ADV_BORDER

static BMfloat calc_wall_collision_time( BallType * ball, BorderType * wall )
{
    BMfloat h, vn, rval, ph, q, t1,t2;
    myvec   dr, r, v;
    rval=0.0;
    if       (wall->pnr==3){
        dr = vec_diff( ball->r, wall->r1 );
        h  = vec_mul( dr, wall->n ) - ball->d/2.0;
        vn = vec_mul( ball->v, wall->n );
        rval = -h/vn;
    } else if(wall->pnr==2){
        /* del all comps par to cylinder */
        dr = vec_diff( wall->r2 ,wall->r1 );
        r = vec_diff( ball->r, wall->r1 );
        r = vec_diff( r, vec_proj(r,dr) );
        v = ball->v;
        v = vec_diff( v, vec_proj(v,dr) );
        ph = vec_mul(v,r)/vec_abssq(v);
        q  = (vec_abssq(r) - ball->d*ball->d/4.0)/vec_abssq(v);
        t1 = -ph+sqrt(ph*ph-q);
        t2 = -ph-sqrt(ph*ph-q);
        /* solve |r+vt|=d/2 */
        rval = (t1<t2)?t1:t2;
    } else if(wall->pnr==1){
        r = vec_diff( ball->r, wall->r1 );
        ph = vec_mul(ball->v,r)/vec_abssq(ball->v);
        q  = (vec_abssq(r) - ball->d*ball->d/4.0)/vec_abssq(v);
        t1 = -ph+sqrt(ph*ph-q);
        t2 = -ph-sqrt(ph*ph-q);
        rval = (t1<t2)?t1:t2;
    }
    if( !inrange_advborder( ball, wall ) ){
        rval=1E20;
    }
    return(rval);
}

#else

static BMfloat calc_wall_collision_time( BallType * ball, BorderType * wall )
{
    BMfloat h, vn;
    myvec   dr;
    dr = vec_diff( ball->r, wall->r );
    h  = vec_mul( dr, wall->n ) - ball->d/2.0;
    vn = vec_mul( ball->v, wall->n );
    return(-h/vn);
}

#endif

static BMfloat calc_ball_collision_time( BallType * b1, BallType * b2 )
{
    BMfloat p, q, vs, rs, t1, t2, ds;
    myvec   dv, dr;
    dv = vec_diff( b1->v, b2->v );
    dr = vec_diff( b1->r, b2->r );
    vs = dv.x*dv.x + dv.y*dv.y + dv.z*dv.z ;
    rs = dr.x*dr.x + dr.y*dr.y + dr.z*dr.z ;
    ds = (b1->d+b2->d)/2.0;
    ds *= ds;
    p  = ( dv.x*dr.x + dv.y*dr.y + dv.z*dr.z )/vs;
    q  = (rs-ds)/vs;
    q  = sqrt(p*p-q);
    t1 = -p + q;
    t2 = -p - q;
    return( (t1<t2)?t1:t2 );
}


#ifdef USE_ADV_BORDER

static void ball_wall_interaction( BallType * ball, BorderType * wall )
{
    myvec dv, dr, n;

    if       (wall->pnr==3){
        dv      = vec_scale(wall->n,vec_mul(wall->n,ball->v)*2.0);
        ball->v = vec_diff(ball->v,dv);
//        fprintf(stderr,"dv=(%f,%f,%f)",dv.x,dv.y,dv.z);
        /* maybe some angular momentum loss has to be implemented here */
        /* ... */
    } else if(wall->pnr==2){
        dr      = vec_diff( wall->r2, wall->r1 );
        n       = vec_diff( ball->r, wall->r1 );
        n       = vec_unit( vec_diff( n, vec_proj(n,dr) ) );
        dv      = vec_scale(n,vec_mul(n,ball->v)*2.0);
        ball->v = vec_diff(ball->v,dv);
    } else if(wall->pnr==1){
        n       = vec_unit( vec_diff( ball->r, wall->r1 ) );
        dv      = vec_scale(n,vec_mul(n,ball->v)*2.0);
        ball->v = vec_diff(ball->v,dv);
    }
}

#else

static void ball_wall_interaction( BallType * ball, BorderType * wall )
{
    myvec dv;
    dv      = vec_scale(wall->n,vec_mul(wall->n,ball->v)*2.0);
    ball->v = vec_diff(ball->v,dv);
    /* maybe some angular momentum loss has to be implemented here */
    /* ... */
}

#endif

#define MU_OMEGA            0.1      /* transmitted ang. mom. ratio (at hit) */

static void ball_ball_interaction( BallType * b1, BallType * b2 )
{
    BallType b1s, b2s;  /* balls in coord system of b1 */
    myvec dvec, duvec, dv2, dw1, dw2;

    dvec = vec_diff(b1->r,b2->r);
    b1s=*b1; b2s=*b2;
    b2s.v.x-=b1s.v.x;   b2s.v.y-=b1s.v.y;   b2s.v.z-=b1s.v.z;
    b1s.v.x=0.0;        b1s.v.y=0.0;        b1s.v.z=0.0;

    duvec = vec_unit(dvec);
    dv2   = vec_scale(duvec,vec_mul(duvec,b2s.v));
    b2s.v = vec_diff( b2s.v, dv2 );
    b1s.v = vec_scale( dv2, b2s.m/b1s.m );
    b2->v = vec_add( b1->v, b2s.v );
    b1->v = vec_add( b1->v, b1s.v );

    dw1 = vec_scale(b1->w,MU_OMEGA);
    dw2 = vec_scale(b2->w,MU_OMEGA);
    b1->w = vec_diff( vec_diff(b1->w,dw1), vec_scale(dw2,(b2->I)/(b1->I)) );
    b2->w = vec_diff( vec_diff(b2->w,dw2), vec_scale(dw1,(b1->I)/(b2->I)) );
}


#define  D_MIN               1.0E-5      /* 1E-5 = 1/100 mm */
#define  V_MIN               1.0E-2      /* 1E-3 = 1 mm/s */
#define  SLIDE_THRESH_SPEED  1.0E-2      /* 1E-2 = 1 cm/s */


static int ball_in_hole(BallType *ball, BordersType *borders)
{
    int i;
    for(i=0;i<borders->holenr;i++){
        if( vec_abs(vec_diff(borders->hole[i].pos,ball->r)) <
            borders->hole[i].r/* - ball->d/2.0*/ ){
            return 1;
        }
    }
    return 0;
}


int ball_outside( BallType *ball, BordersType *borders )
{
    int rval=0;
    int i;
    for(i=0;i<borders->nr;i++){
        rval = rval || BALL_WALL_DIST_I(ball[0],borders->border[i])<0.0;
    }
    return rval;
}


void remove_balls_from_game( BallsType *balls, BordersType *borders )
{
    int i;
    for(i=0;i<balls->nr;i++){
//        if( ball_outside(&balls->ball[i],borders) ){
        if( ball_in_hole(&balls->ball[i], borders) ){
            balls->ball[i].in_game=0;
            if( balls->ball[i].nr==0 ) move_log.out_white=1;
            if( balls->ball[i].nr==8 ) move_log.out_black=1;
            if( balls->ball[i].nr<8 && balls->ball[i].nr!=0 ) move_log.out_full++;
            if( balls->ball[i].nr>8 && balls->ball[i].nr!=0 ) move_log.out_half++;
        }
    }
}


static int proceed_dt_euler(BallsType *balls, BordersType *borders, BMfloat dt, int depth)
/* this one does not remove fallen balls */
{
    myvec dx;
    BMfloat dphi, dt1;
    int i,j,k;
    int rnd[100];
    int collided=0;

//    if (depth>500) return 0;

//    fprintf(stderr,"proceed_dt_euler: depth=%d\n",depth);

    /* move all balls */
    for(i=0;i<balls->nr;i++){
        /* translate ball */
        dx = vec_scale(balls->ball[i].v,dt);
        balls->ball[i].r = vec_add(balls->ball[i].r,dx);
        /* perform rotation */
        dphi = vec_abs(balls->ball[i].w)*dt;
        //        rot_ax( balls->ball[i].w, balls->ball[i].b, 3, -dphi );
        if( dphi > 1.0E-30 )
            rot_ax( balls->ball[i].w, balls->ball[i].b, 3, dphi );
    }
    calc_rand_table(rnd,balls->nr);
    /* checks */
    for(i=0;i<balls->nr;i++) if(balls->ball[i].in_game){ /* check for one collision */
        /* check wall collisions */
        for(j=0;j<borders->nr;j++){
//            fprintf(stderr,"1:bwdist=%f\n",BALL_WALL_DIST(balls->ball[i],borders->border[j]));
            if( BALL_WALL_DIST(balls->ball[i],borders->border[j]) < balls->ball[i].d/2.0 && !ball_in_hole(&(balls->ball[i]),borders) ){
//                fprintf(stderr,"found ball-border collision\n");
                dt1=calc_wall_collision_time(&balls->ball[i],&borders->border[j]);
                /* dt1 should be negative */
//#endif
//                fprintf(stderr,"dt1=%f j=%d\n",dt1,j);
                if( dt1>0.0 || fabs(dt)>dt ){
                    if( dt1>0.0 ){
//                        fprintf(stderr,"wall: dt1>0 !!!\n");
                    }
                    if( fabs(dt)>dt ){
//                        fprintf(stderr,"wall: |dt1| > dt\n");
                    }
                } else {
                    proceed_dt_euler(balls,borders,dt1,depth+1); /* move back to coll. */
//                    fprintf(stderr,"bwdist=%f\n",fabs(BALL_WALL_DIST(balls->ball[i],borders->border[j])));
                    if ( fabs(BALL_WALL_DIST(balls->ball[i],borders->border[j])-balls->ball[i].d/2.0) < 1E-5 /*1E-5 = 1/100 mm*/ ){
                        /* only if ball still interacts */
                        /* (ball may seem to collide with wall but when stepping back out of */
                        /* wall region (wall border)) */
                        ball_wall_interaction(&balls->ball[i],&borders->border[j]);
                        collided=1;
                    }
                    proceed_dt_euler(balls,borders,-dt1,depth+1); /* move rest */
//                    if( collided ) break;
                    /*break;*/ /* no break because of second freeze bug */
                }
            }
        }

        /* check ball collision */
        if (!collided) for(j=0,k=rnd[0];j<balls->nr;j++,k=rnd[j]) if(k!=i){
            if( BALL_BALL_DIST( balls->ball[i], balls->ball[k] ) < (balls->ball[i].d+balls->ball[k].d)/2.0 ){
//                fprintf(stderr,"found ball-ball collision\n");
                dt1=calc_ball_collision_time(&balls->ball[i],&balls->ball[k]); /* dt1 should be negative */
                if( dt1>0.0 || fabs(dt)>dt ){
                    if( dt1>0.0 ){
//                        fprintf(stderr,"ball: dt1>0 !!!\n");
                    }
                    if( fabs(dt)>dt ){
//                        fprintf(stderr,"ball: |dt1| > dt\n");
                    }
                } else {
                    proceed_dt_euler(balls,borders,dt1,depth+1); /* move back to coll. */
                    if ( fabs( BALL_BALL_DIST( balls->ball[i], balls->ball[k] ) - (balls->ball[i].d+balls->ball[k].d)/2.0 ) < 1E-5 /*1E-5 = 1/100 mm*/ ){
                        /* only if balls still interact */
                        ball_ball_interaction(&balls->ball[i],&balls->ball[k]);
                    }
                    proceed_dt_euler(balls,borders,-dt1,depth+1); /* move rest */
                    break;
                }
            }
        }
    }
//    fprintf(stderr,"proceed_dt_euler: end: depth=%d\n",depth);
    return(0);
}


static void proceed_dt_only(BallsType *balls, BordersType *borders, BMfloat dt)
{
    myvec dx;
    BMfloat dphi;
    int i;

    for(i=0;i<balls->nr;i++){
        /* translate ball */
        dx = vec_scale(balls->ball[i].v,dt);
        balls->ball[i].r = vec_add(balls->ball[i].r,dx);
        /* perform rotation */
        dphi = vec_abs(balls->ball[i].w)*dt;
        //        rot_ax( balls->ball[i].w, balls->ball[i].b, 3, -dphi );
        if( dphi > 1.0E-30 )
            rot_ax( balls->ball[i].w, balls->ball[i].b, 3, dphi );
    }
}


static int wall_dv_pos( BallType * ball, BorderType * wall )
/* returns 1 if border and ball strobe away fromeach other, 0 else */
{
    if       ( wall->pnr == 3 ){
        return( vec_mul(ball->v,wall->n) > 0.0 );
    } else if( wall->pnr == 2 ){
        return(
                vec_mul(ball->v,
                        vec_ncomp(
                                  vec_diff(ball->r,wall->r1),
                                  vec_diff(wall->r2,wall->r1)
                                 )
                       ) > 0.0
              );
    } else if( wall->pnr == 1 ){
        return( vec_mul(vec_diff(ball->r,wall->r1),ball->v) > 0.0 );
    }
    return 1;
}

static int ball_dv_pos( BallType * b1, BallType * b2 )
/* returns 1 if balls strobe away fromeach other, 0 else */
{
    return( vec_mul(vec_diff(b2->r,b1->r),vec_diff(b2->v,b1->v)) > 0.0 );
}


static int proceed_dt_euler_new(BallsType *balls, BordersType *borders, BMfloat dt, int depth)
/* this one does not remove fallen balls */
{
    BMfloat dt1, dtmin;
    int i,j;
//    int rnd[100];
//    int collided=0;
//    int overlapnr,walloverlap,ball1overlap,ball2overlap;
    int colltype, collnr, collnr2;
#define COLLTYPE_WALL 1
#define COLLTYPE_BALL 2
#define COLLTYPE_NONE 0

//    if (depth>500) return 0;

//    fprintf(stderr,"proceed_dt_euler: depth=%d\n",depth);

    /* move all balls */
//    fprintf(stderr,"proceed_dt_euler: dt=%f depth=%d\n",dt, depth);
    proceed_dt_only(balls,borders,dt);

    /* checks */
    colltype=COLLTYPE_NONE; collnr=-1; collnr2=-1;
    dtmin=0.0;
    for(i=0;i<balls->nr;i++) if(balls->ball[i].in_game){ /* check for one collision */
        /* check wall collisions */
        for(j=0;j<borders->nr;j++){
            dt1=calc_wall_collision_time(&balls->ball[i],&borders->border[j]);
            if( dt1<dtmin && dt1>-dt ){
                if( !wall_dv_pos(&balls->ball[i],&borders->border[j]) ){ /* dont strobe apart */
                    dtmin=dt1; colltype=COLLTYPE_WALL; collnr=j; collnr2=i;
                }
            }
        }
        /* check ball collisions */
        for(j=0;j<balls->nr;j++) if( j!=i && balls->ball[j].in_game ){
            dt1=calc_ball_collision_time(&balls->ball[i],&balls->ball[j]); /* dt1 should be negative */
            if( dt1<dtmin && dt1>-dt ){
                if( !ball_dv_pos(&balls->ball[j],&balls->ball[i]) ){ /* dont strobe apart */
                    dtmin=dt1; colltype=COLLTYPE_BALL; collnr=j; collnr2=i;
                }
            }
        }
    }

    if( colltype==COLLTYPE_NONE ){
    } else if( colltype==COLLTYPE_WALL ){
//        fprintf(stderr,"proceed_dt_euler: wall collision dtmin=%f\n",dtmin);
//        if( !wall_dv_pos(&balls->ball[collnr2],&borders->border[collnr]) ){ /* dont strobe apart */
            proceed_dt_only(balls,borders,dtmin);
            ball_wall_interaction(&balls->ball[collnr2],&borders->border[collnr]);
            proceed_dt_euler_new(balls,borders,-dtmin,depth+1);
//        }
    } else if( colltype==COLLTYPE_BALL ){
//        fprintf(stderr,"proceed_dt_euler: ball collision\n");
//        if( !ball_dv_pos(&balls->ball[collnr],&balls->ball[collnr2]) ){ /* dont strobe apart */
            proceed_dt_only(balls,borders,dtmin);
            ball_ball_interaction(&balls->ball[collnr],&balls->ball[collnr2]);
            proceed_dt_euler_new(balls,borders,-dtmin,depth+1);
//        }
    }

    return(0);
}


#define GRAVITY            9.81      /* m/s^2 */
#define MU_ROLL            0.01      /* table  roll-friction const */
#define MU_SLIDE           0.1       /* table slide-friction const */
#define SPOT_R             6.0e-3    /* 3mm radius der auflageflaeche */
//#define  OMEGA_MIN           M_PI/8.0   /* 22.5/s */
#define OMEGA_MIN         SLIDE_THRESH_SPEED/SPOT_R   /* 22.5/s */


myvec perimeter_speed( BallType * ball )
{
    return(
            vec_cross(
                      ball->w,
                      vec_xyz(0.0,0.0,-ball->d/2.0)
                     )
           );
}



int proceed_dt(BallsType *balls, BordersType *borders, BMfloat dt)
{
    int i;
    int balls_moving=0;
    myvec accel, waccel, uspeed, uspeed_eff, uspeed2, uspeed_eff2,
          fricaccel, fricmom, rollmom, totmom;
    double uspeed_eff_par, uspeed_eff2_par;

    /* timestep wit actual speeds, omegas,... */
    proceed_dt_euler_new(balls, borders, dt, 0);

    /* calc new accelerations and speeds */
    for(i=0;i<balls->nr;i++) if(balls->ball[i].in_game){

        /* check if balls still moving */
        if( vec_abs(balls->ball[i].v)!=0 || vec_abs(balls->ball[i].w)!=0 ){
            balls_moving=1;
        }

        /* calc accel 3D */
        accel  = vec_xyz(0.0,0.0,0.0); // init acceleration

        /* absolute and relative perimeter speed */
        uspeed = perimeter_speed(&balls->ball[i]);
        uspeed_eff = vec_add(uspeed,balls->ball[i].v);


        if( vec_abs( uspeed_eff ) > SLIDE_THRESH_SPEED ){  /* if sliding */

//            if(i==0) fprintf(stderr,"sliding\n");

            /* acc caused by friction */
            fricaccel = vec_scale( vec_unit(uspeed_eff), -MU_SLIDE*GRAVITY );
            accel = vec_add(accel, fricaccel);
            /* angular acc caused by friction */
            fricmom = vec_scale( vec_cross(fricaccel, vec_xyz(0.0,0.0,-balls->ball[i].d/2.0)), balls->ball[i].m );
            waccel = vec_scale( fricmom,-1.0/balls->ball[i].I );

            /* perform accel */
            balls->ball[i].w = vec_add( balls->ball[i].w, vec_scale(waccel,dt) );
            balls->ball[i].v = vec_add( balls->ball[i].v, vec_scale(accel,dt) );
            uspeed2 = perimeter_speed(&balls->ball[i]);
            uspeed_eff2 = vec_add(uspeed2,balls->ball[i].v);
            /* if uspeed_eff passes 0 */
            //            if( vec_angle(uspeed_eff2,uspeed_eff) > M_PI/2.0 ){
            uspeed_eff_par  = vec_mul( uspeed_eff,  vec_diff(uspeed_eff,uspeed_eff2) );
            uspeed_eff2_par = vec_mul( uspeed_eff2, vec_diff(uspeed_eff,uspeed_eff2) );
            if( vec_ndist(vec_null(),uspeed_eff,uspeed_eff2) <= SLIDE_THRESH_SPEED &&
                ( (uspeed_eff_par > 0.0 && uspeed_eff2_par < 0.0) ||
                  (uspeed_eff2_par > 0.0 && uspeed_eff_par < 0.0) )   ){
                /* make rolling if uspeed_eff passed 0 */
/*                myvec dummy;
                dummy = vec_ncomp(vec_xyz(0.0,0.0,-balls->ball[i].d/2.0),balls->ball[i].w);
                balls->ball[i].w = vec_scale(vec_cross(balls->ball[i].v,dummy),
                                             1.0/vec_abssq(dummy));*/
                balls->ball[i].v = vec_cross( balls->ball[i].w,
                                              vec_xyz(0.0,0.0,balls->ball[i].d/2.0) );
                if(i==0) fprintf(stderr,"made rolling\n");
            }
            if( vec_abs(balls->ball[i].w) < OMEGA_MIN &&
                vec_abs(balls->ball[i].v) < SLIDE_THRESH_SPEED ){
                balls->ball[i].v = vec_xyz( 0.0, 0.0, 0.0 );
                balls->ball[i].w = vec_xyz( 0.0, 0.0, 0.0 );
            }

        } else {                                             /* rolling forces */

            fricmom = vec_xyz(0.0,0.0,0.0);

//            if(i==0) fprintf(stderr,"rolling\n");

//            if( fabs(vec_mul(vec_xyz(0.0,0.0,SPOT_R),balls->ball[i].w)) > SLIDE_THRESH_SPEED ){
            if( fabs(vec_mul(vec_xyz(0.0,0.0,1.0),balls->ball[i].w)) > OMEGA_MIN ){
                fricmom = vec_add(fricmom,
                                  vec_scale(
                                            vec_unit(vec_xyz(0.0,0.0,balls->ball[i].w.z)),
                                            MU_SLIDE*balls->ball[i].m*GRAVITY*SPOT_R
                                           )
                                 );
            } else {
            }


//            #define ROLL_MOM_R MU_ROLL*balls->ball[i].d/5.0   /* wirkabstand von rollwid.-kraft */
            /* wirkabstand von rollwid.-kraft */
            #define ROLL_MOM_R MU_ROLL*balls->ball[i].I/balls->ball[i].m/balls->ball[i].d

            rollmom = vec_cross(
                                vec_xyz(0.0,0.0,balls->ball[i].m*GRAVITY * ROLL_MOM_R),
                                vec_unit(balls->ball[i].v)
                               );

            totmom = vec_add(fricmom,rollmom);
            waccel = vec_scale( totmom,-1.0/balls->ball[i].I );

            balls->ball[i].w = vec_add( balls->ball[i].w, vec_scale(waccel,dt) );
            /* align v with w to assure rolling */
            balls->ball[i].v = vec_cross( balls->ball[i].w,
                                          vec_xyz(0.0,0.0,balls->ball[i].d/2.0) );

/*            if( vec_abs(balls->ball[i].v) < SLIDE_THRESH_SPEED ){
                balls->ball[i].v = vec_xyz( 0.0, 0.0, 0.0 );
                balls->ball[i].w = vec_xyz( 0.0, 0.0, 0.0 );
            }*/

            if( vec_abs(balls->ball[i].w) < OMEGA_MIN &&
                vec_abs(balls->ball[i].v) < SLIDE_THRESH_SPEED ){
                balls->ball[i].v = vec_xyz( 0.0, 0.0, 0.0 );
                balls->ball[i].w = vec_xyz( 0.0, 0.0, 0.0 );
            }

        }
        /* calc waccel 3D */
//        waccel = vec_scale( vec_add(vec_scale(fricmom,+1.0),rollmom),-1.0/balls->ball[i].I*dt);
//        waccel = vec_scale( vec_proj(fricmom,balls->ball[i].w),
//                            -1.0/balls->ball[i].I*dt );
//        waccel = vec_scale( rollmom,-1.0/balls->ball[i].I*dt);
//        waccel = vec_scale(balls->ball[i].w,-0.1*dt); // fix this

        /* w - omega */
//        balls->ball[i].w = vec_add( balls->ball[i].w, vec_scale(waccel,dt) );
//        if( vec_abs(balls->ball[i].w) < OMEGA_MIN )
//            balls->ball[i].w = vec_xyz( 0.0, 0.0, 0.0 );
        /* v - speed */
/*        balls->ball[i].v = vec_add( balls->ball[i].v, vec_scale(accel,dt) );
        if( vec_abs(balls->ball[i].v) < SLIDE_THRESH_SPEED ){
            balls->ball[i].v = vec_xyz( 0.0, 0.0, 0.0 );
            balls->ball[i].w = vec_xyz( 0.0, 0.0, 0.0 );
        }*/
    }

    remove_balls_from_game( balls, borders );

    return balls_moving;
}
